#include "PS2Mouse.h"
// PS2 mouse pins
#define DATA_PIN 2
#define CLOCK_PIN 3

// jotstick pins
#define trigPin 4     // discharge timing capacitors
#define axPin 5
#define ayPin 6
#define bxPin 7
#define byPin 8
#define ab1Pin 9
#define ab2Pin 10
#define bb1Pin 11
#define bb2Pin 12
#define joyM 2        // joystick deadzone

int x,y,w,stat,stato;
unsigned long timeZero, timeWait;
signed long axt,ayt,bxt,byt;
signed long axto,ayto,bxto,byto;
int p,ab,bb, abo,bbo;
  
PS2Mouse mouse(CLOCK_PIN, DATA_PIN);

void setup() {
  Serial.begin(19200);
  pinMode(axPin, INPUT);
  pinMode(ayPin, INPUT);
  pinMode(bxPin, INPUT);
  pinMode(byPin, INPUT);
  pinMode(ab1Pin, INPUT_PULLUP);
  pinMode(ab2Pin, INPUT_PULLUP);
  pinMode(bb1Pin, INPUT_PULLUP);
  pinMode(bb2Pin, INPUT_PULLUP);
  pinMode(trigPin, OUTPUT);
  digitalWrite(trigPin, LOW);
  
  mouse.initialize();
}

void loop() {
    MouseData data = mouse.readData();
    stat = data.status;
    x = data.position.x;
    y = data.position.y;
    w = data.wheel;
    if(x != 0 || y != 0 || stato != stat){
    Serial.write(stat);
    Serial.write(x);
    Serial.write(y);
    Serial.write(w);
    stato = stat;
  }
  delay(3);
  axt = 0;
  ayt = 0;
  bxt = 0;
  byt = 0;
  
  digitalWrite(trigPin, HIGH); // discharge the capacitors
  delayMicroseconds(100);
  digitalWrite(trigPin, LOW);
  timeZero = micros();
  timeWait = timeZero + 2050; // set timeout

  while (micros() < timeWait) {
    if (axt == 0 && digitalRead(axPin) == LOW) {
      axt = micros()-timeZero;
    }
    if (ayt == 0 && digitalRead(ayPin) == LOW){
      ayt = micros()-timeZero;
    }
    if (bxt == 0 && digitalRead(bxPin) == LOW){
      bxt = micros()-timeZero;
    }
    if (byt == 0 && digitalRead(byPin) == LOW){
      byt = micros()-timeZero;
    }
  }
  /* nano only has 4 uS resolution so divide by 4
     add high byte bits to button byte
    2 bits to identify mouse or joystick, 2 bits for x axis, 
    2 bits for y axis 2 bits for buttons
  */
  axt = axt / 4;
  ayt = ayt / 4;
  bxt = bxt / 4;
  byt = byt / 4;
  ab = (highByte(axt) & 3 )*16 + (highByte(ayt) & 3 )*4 + 64;
  ab = ab + (1-digitalRead(ab2Pin)) *2;
  ab = ab + 1-digitalRead(ab1Pin);
  
  bb = (highByte(bxt) & 3 )*16 + (highByte(byt) & 3 )*4 + 128;
  bb = bb + (1-digitalRead(bb2Pin)) *2;
  bb = bb + 1-digitalRead(bb1Pin);

  if (ab != abo || abs(axt-axto) > joyM || abs(ayt-ayto)> joyM) {
    Serial.write(ab);
    Serial.write(lowByte(axt));
    Serial.write(lowByte(ayt));
    Serial.write('A');
    abo = ab;
    axto = axt;
    ayto = ayt;
  }
  delay(3);    // give the micromite time to digest
  if (bb != bbo || abs(bxt-bxto) > joyM || abs(byt-byto)> joyM) {
    Serial.write(bb);
    Serial.write(lowByte(bxt));
    Serial.write(lowByte(byt));
    Serial.write('B');
    bbo = bb;
    bxto = bxt;
    byto = byt;
  }
  
  delay(15);  
}
